@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import anyio
+import click
 import contextvars as cv
 import logging
 import traceback
@@ -11,11 +12,12 @@ from starlette.routing import BaseRoute
 from starlette.types import ASGIApp, Receive, Scope, Send
 
 from kupala.config import Config
+from kupala.console.application import ConsoleApplication
 from kupala.container import Container, Resolver
 from kupala.contracts import ContextProcessor, Invoker
 from kupala.dotenv import DotEnv
 from kupala.exceptions import ErrorHandler, ExceptionMiddleware
-from kupala.middleware import MiddlewareStack
+from kupala.middleware import Middleware, MiddlewareStack
 from kupala.requests import Request
 from kupala.routing import Router, Routes
 
@@ -30,12 +32,14 @@ class Kupala:
         env_prefix: str = '',
         environment: str = None,
         config: t.Mapping = None,
-        dotenv: t.List[str] = None,
+        dotenv: list[str] = None,
         template_dirs: list[str] = None,
         providers: t.Iterable['Provider'] = None,
-        routes: t.Union[Routes, t.List[BaseRoute]] = None,
-        context_processors: t.List[ContextProcessor] = None,
-        error_handlers: t.Dict[t.Union[t.Type[Exception], int], ErrorHandler] = None,
+        routes: t.Union[Routes, list[BaseRoute]] = None,
+        context_processors: list[ContextProcessor] = None,
+        commands: list[click.Command] = None,
+        error_handlers: dict[t.Union[t.Type[Exception], int], t.Optional[ErrorHandler]] = None,
+        middleware: t.Union[list[Middleware], MiddlewareStack] = None,
     ) -> None:
         self.lifespan: t.List[t.Callable[[Kupala], t.AsyncContextManager]] = []
         self.routes = Routes(list(routes) if routes else [])
@@ -43,12 +47,13 @@ class Kupala:
         self.dotenv = DotEnv(dotenv or ['.env.defaults', '.env'], env_prefix)
         self.env = self.dotenv.str('APP_ENV', 'production') if environment is None else environment
         self.debug = self.dotenv.bool('APP_DEBUG', False) if debug is None else debug
-        self.middleware = MiddlewareStack()
+        self.middleware = MiddlewareStack(list(middleware or []))
         self.providers = providers or []
         self.context_processors: t.List[ContextProcessor] = context_processors or []
         self.error_handlers = error_handlers or {}
         self.services = Container()
         self.template_dirs = template_dirs or []
+        self.commands = commands or []
 
         self._asgi_app: t.Optional[ASGIApp] = None
         self._router: t.Optional[Router] = None
@@ -82,11 +87,15 @@ class Kupala:
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         assert scope["type"] in {"http", "websocket", "lifespan"}
-        scope["app"] = self
         set_current_application(self)
 
+        if self._asgi_app is None:
+            self._asgi_app = self._create_app()
+
         request_container = Container(parent=self.services)
+        scope["app"] = self
         scope['resolver'] = request_container
+        scope["router"] = self._router
 
         if scope["type"] == "lifespan":
             await self.lifespan_handler(scope, receive, send)
@@ -98,16 +107,18 @@ class Kupala:
             scope["request"] = request
             self._request.set(request)
 
-        if self._asgi_app is None:
-            self._asgi_app = self._create_app()
-
         self._request_container.set(request_container)
-        with request_container.change_context():
+        with request_container.change_context({}):
+            assert self._asgi_app
             await self._asgi_app(scope, receive, send)
 
-    def cli(self) -> None:
+    def cli(self) -> int:
         set_current_application(self)
         self.bootstrap()
+        app = ConsoleApplication(self.services, self.commands)
+
+        with self.services.change_context({}):
+            return app.run()
 
     def bootstrap(self) -> None:
         for provider in self.providers:
