@@ -8,13 +8,13 @@ import typing
 import typing as t
 import uuid
 from deesk.storage import Storage
-from imia import UserToken
+from imia import UserLike, UserToken
 from starlette import datastructures as ds, requests
 from starlette.requests import empty_receive, empty_send
 from starlette.types import Receive, Scope, Send
 from starsessions import Session
 
-from .storages import Storages
+from kupala.disks.storages import Storages
 
 if t.TYPE_CHECKING:
     from .application import Kupala
@@ -118,16 +118,23 @@ class UploadFile(ds.UploadFile):
             request=request,
         )
 
-    async def save(self, directory: t.Union[str, os.PathLike], filename: str = None, disk: str = 'default') -> str:
-        uploaded_filename = self.filename
-        prefix = str(uuid.uuid4().fields[-1])[:5]
+    async def save(self, directory: t.Union[str, os.PathLike], filename: str = None, disk: str = None) -> str:
+        uploaded_filename = pathlib.Path(self.filename)
         extension = pathlib.Path(uploaded_filename).suffix
+        base_name = os.path.basename(uploaded_filename).replace(extension, '')
+        prefix = str(uuid.uuid4().fields[-1])[:5]
         if not extension:
-            extension = mimetypes.guess_extension(self.content_type) or 'bin'
-        suggested_filename = f'{prefix}_{uploaded_filename}.{extension}'
+            extension = '.' + (mimetypes.guess_extension(self.content_type) or 'bin')
+        extension = extension.lstrip('.')
+        suggested_filename = f'{prefix}_{base_name}.{extension}'
 
         file_path = os.path.join(directory, filename or suggested_filename)
-        storage: Storage = self.request.app.resolve(Storages).get(disk)
+
+        storage: Storage
+        if disk:
+            storage = self.request.app.resolve(Storages).get(disk)
+        else:
+            storage = self.request.app.resolve(Storages).get_default_disk()
         await storage.put(file_path, await self.read())
         return file_path
 
@@ -175,6 +182,10 @@ class Request(requests.Request):
         assert "auth" in self.scope, "AuthenticationMiddleware must be installed to access request.auth"
         return self.scope["auth"]
 
+    @property
+    def user(self) -> UserLike:
+        return self.auth.user
+
     @property
     def wants_json(self) -> bool:
         """Test if request sends Accept header
@@ -248,7 +259,7 @@ class Request(requests.Request):
         for pattern in patterns:
             if pattern == self.url.path:
                 return True
-            if re.match(pattern, self.url.path):
+            if re.search(pattern, self.url.path):
                 return True
         return False
 
@@ -272,3 +283,6 @@ class Request(requests.Request):
     async def files(self) -> FilesData:
         data = await self.form()
         return FilesData(FormData([(k, v) for k, v in data.multi_items() if isinstance(v, UploadFile)]))
+
+    def __repr__(self) -> str:
+        return f'<{self.__class__.__name__}: {self.method} {self.url}>'
