@@ -5,12 +5,10 @@ import typing as t
 import urllib.parse
 from starlette import responses
 from starlette.background import BackgroundTask
-from starlette.datastructures import UploadFile
 from starlette.types import Receive, Scope, Send
 from urllib.parse import quote
 
 from kupala import json
-from kupala.constants import REDIRECT_INPUT_DATA_SESSION_KEY
 from kupala.json import JSONEncoder
 from kupala.requests import Request
 
@@ -103,7 +101,7 @@ class RedirectResponse(responses.RedirectResponse):
         status_code: int = 302,
         headers: dict = None,
         *,
-        input_data: t.Mapping = None,
+        capture_input: bool = False,
         flash_message: str = None,
         flash_category: str = "info",
         path_name: str = None,
@@ -117,9 +115,9 @@ class RedirectResponse(responses.RedirectResponse):
         self._url = url
         self._flash_message = flash_message
         self._flash_message_category = flash_category
-        self._input_data = input_data
         self._path_name = path_name
         self._path_params = path_params
+        self._capture_input = capture_input
 
         assert url or path_name, 'Either "url" or "path_name" argument must be passed.'
 
@@ -129,27 +127,25 @@ class RedirectResponse(responses.RedirectResponse):
         self._flash_message_category = category
         return self
 
-    def with_error(self: RT, message: str, input_data: t.Mapping = None) -> RT:
+    def with_error(self: RT, message: str, capture_input: bool = True) -> RT:
         response = self.flash(message, category='error')
-        if input_data:
-            response = self.with_input(input_data)
+        if capture_input:
+            response = self.with_input()
         return response
 
     def with_success(self: RT, message: str) -> RT:
         return self.flash(message, category='success')
 
-    def with_input(self: RT, input_data: t.Mapping) -> RT:
+    def with_input(self: RT) -> RT:
         """Redirect with form input data. Uploaded files will be removed from data."""
-        self._input_data = {k: v for k, v in input_data.items() if not isinstance(v, UploadFile)}
+        self._capture_input = True
+        # self._input_data = {k: v for k, v in input_data.items() if not isinstance(v, UploadFile)}
         return self
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         if self._flash_message and 'flash_messages' in scope:
             scope['flash_messages'].add(self._flash_message, self._flash_message_category)
 
-        if self._input_data and 'session' in scope:
-            scope['session'][REDIRECT_INPUT_DATA_SESSION_KEY] = self._input_data
-
         if self._path_name:
             url = scope['request'].url_for(self._path_name, **(self._path_params or {}))
         else:
@@ -157,6 +153,9 @@ class RedirectResponse(responses.RedirectResponse):
 
         self.headers["location"] = quote(str(url), safe=":/%#?=@[]!$&'()*+,;")
 
+        if 'session' in scope and self._capture_input:
+            await scope['request'].remember_form_data()
+
         return await super().__call__(scope, receive, send)
 
 
@@ -171,7 +170,7 @@ class GoBackResponse(RedirectResponse):
         request: Request,
         flash_message: str = None,
         flash_category: str = 'info',
-        input_data: t.Any = None,
+        capture_input: bool = False,
         status_code: int = 302,
     ) -> None:
         redirect_to = request.headers.get('referer', '/')
@@ -181,7 +180,7 @@ class GoBackResponse(RedirectResponse):
         super().__init__(
             redirect_to,
             status_code=status_code,
-            input_data=input_data,
+            capture_input=capture_input,
             flash_message=flash_message,
             flash_category=flash_category,
         )
